#==============================================================================
#
#  $Id: addrwin.py,v 1.5 2001/03/18 17:17:39 mike Exp $
#
"""
   This will eventually house all of the front end stuff for address books.
"""
#
#  Copyright (C) 1999 Michael A. Muller
#
#  Permission is granted to use, modify and redistribute this code,
#  providing that the following conditions are met:
#
#  1) This copyright/licensing notice must remain intact.
#  2) If the code is modified and redistributed, the modifications must 
#  include documentation indicating that the code has been modified.
#  3) The author(s) of this code must be indemnified and held harmless
#  against any damage resulting from the use of this code.
#
#  This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
#  warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#
#  $Log: addrwin.py,v $
#  Revision 1.5  2001/03/18 17:17:39  mike
#  Added support for editing groups.
#
#  Revision 1.4  2000/07/16 17:46:45  mike
#  Implemented the creation of address groups.
#
#  Revision 1.3  2000/05/25 01:22:40  mike
#  Changed call to queryYesNo() as per change in mainwin.
#
#  Revision 1.2  2000/04/30 21:07:22  mike
#  Added initial address book window.
#
#  Revision 1.1.1.1  2000/04/02 19:09:03  mike
#  Spug Mailer and libraries.
#
#
#==============================================================================

import sys
sys.path.insert(0, '../..')

from spug.curses.KeyEvent import KeyEvent
from spug.curses.ModalWindow import ModalWindow
from spug.curses.Label import Label
from spug.curses.Entryfield import Entryfield
from spug.curses.Button import Button
from spug.curses.Listbox import Listbox

import main
from addrbk import AddressEntry, GroupEntry

class EditAddress(ModalWindow):
   
   def __init__(self, x0, y0, address):
      ModalWindow.__init__(self, x0, y0, 35, 7)
      self.box()
      self.lastNameLbl = Label(self, 1, 1, 'Last Name:')
      self.lastNameFld = Entryfield(self, 13, 1, 20, 20, address.lastName)
      self.firstNameLbl = Label(self, 1, 2, 'First Name:')
      self.firstNameFld = Entryfield(self, 13, 2, 20, 20, address.firstName)
      self.nicknameLbl = Label(self, 1, 3, 'Nickname:')
      self.nicknameFld = Entryfield(self, 13, 3, 20, 20, address.nickname)
      self.emailAddrLbl = Label(self, 1, 4, 'Address:')
      self.emailAddrFld = Entryfield(self, 13, 4, 20, 80, address.emailAddress)
      self.okBtn = Button(self, 1, 5, 'OK', self.okPressed)
      self.cancelBtn = Button(self, 6, 5, 'Cancel', self.cancelPressed)
      self.setFocus(self.lastNameFld)
      self.refresh()
      
      self.address = address

   def okPressed(self):
      if hasattr(self, 'okBtn'):
         del self.okBtn, self.cancelBtn
      
      # create a new address entry to test against
      addr = AddressEntry(self.nicknameFld.getText(),
                          self.lastNameFld.getText(),
                          self.firstNameFld.getText(),
                          self.emailAddrFld.getText()
                          )
      
      # check for a nickname
      if not addr.nickname:
         main.mainWin.showStatus('You must specify a nickname')

      # check for an e-mail address      
      elif not addr.emailAddress:
         main.mainWin.showStatus('You must specify an e-mail address')
      
      # if this is a new entry or the nickname was changed, check to see
      # if there is a conflict with an existing item of the same nickname
      elif (not self.address.nickname or 
            addr.nickname != self.address.nickname) and \
           main.addressBook.has_key(addr.nickname):
         main.mainWin.showStatus('There is another entry with a nickname '
                                  'of %s' % addr.nickname
                                 )

      else:         
         self.address.lastName = addr.lastName
         self.address.firstName = addr.firstName
         self.address.nickname = addr.nickname
         self.address.emailAddress = addr.emailAddress
         self.result = 1
         self.terminate()

   def cancelPressed(self):
      if hasattr(self, 'okBtn'):
         del self.okBtn, self.cancelBtn
      self.result = 0
      self.terminate()

   def processEvent(self, event):
      if not ModalWindow.processEvent(self, event):
         if isinstance(event, KeyEvent) and event.code == 10:
            self.okPressed()

class EditGroup(ModalWindow):
   
   def __init__(self, x0, y0, group):
      ModalWindow.__init__(self, x0, y0, 33, 5)
      self.box()
      self.nicknameLbl = Label(self, 1, 1, 'Nickname:')
      self.nicknameFld = Entryfield(self, 12, 1, 20, 20, group.nickname)
      self.fullNameLbl = Label(self, 1, 2, 'Full Name:')
      self.fullNameFld = Entryfield(self, 12, 2, 20, 40, group.fullName)
      self.okBtn = Button(self, 1, 3, 'OK', self.okPressed)
      self.cancelBtn = Button(self, 6, 3, 'Cancel', self.cancelPressed)
      self.setFocus(self.nicknameFld)
      self.refresh()
      
      self.group = group
   
   def okPressed(self):
      if hasattr(self, 'okBtn'):
         del self.okBtn, self.cancelBtn
      
      # create a new group instance to test against      
      group = GroupEntry(self.nicknameFld.getText(),
                         self.fullNameFld.getText(),
                         []
                         )
      
      if not group.nickname:
         main.mainWin.showStatus('You must specify a nickname')
      elif not group.fullName:
         main.mainWin.showStatus('You must specify a fullname')
      
      # again, check for nickname conflicts
      elif (not self.group.nickname or 
            group.nickname != self.group.nickname) and \
           main.addressBook.has_key(group.nickname):
         main.mainWin.showStatus('There is another entry with a nickname '
                                  'of %s' % group.nickname
                                 )
      else:
         self.group.nickname = group.nickname
         self.group.fullName = group.fullName
         self.result = 1
         self.terminate()
      
   def cancelPressed(self):
      if hasattr(self, 'okBtn'):
         del self.okBtn, self.cancelBtn
      self.result = 0
      self.terminate()

   def processEvent(self, event):
      if not ModalWindow.processEvent(self, event):
         if isinstance(event, KeyEvent) and event.code == 10:
            self.okPressed()

class AddressListbox(Listbox):
   
   def _itemAsString(self, item):
      if item.firstName:
         return '%-15s %-15s %-15s %-31s' % \
            (item.lastName, item.firstName, item.nickname, item.emailAddress)
      else:
         return '%-31s %-15s %-31s' % \
            (item.lastName, item.nickname, item.emailAddress)

class GroupListbox(Listbox):

   def __init__(self, parent, x, y, width, height, items):
      Listbox.__init__(self, parent, x, y, width, height, items)
      self.__parent = parent

   def _selectionChanged(self, index, item):
      self.__parent._selectedGroupChanged(item)
   
   def _itemAsString(self, group):
      if group.fullName:
         return '%s - %s' % (group.fullName, group.nickname)
      else:
         return group.nickname

class AddrList(ModalWindow):

   def __init__(self, x0, y0, width, height):
      ModalWindow.__init__(self, x0, y0, width, height)
      lbHeight = height * 3 / 4
      addresses = main.addressBook.getAllAddresses()
      addresses.sort()
      self.addresses = AddressListbox(self, 0, 0, width, lbHeight, 
                                      addresses
                                      )
      
      # groups listbox
      groupWidth = width / 2
      self.groupsLbl = Label(self, 0, lbHeight, "Groups:")
      groups = main.addressBook.getAllGroups()
      self.groups = GroupListbox(self, 0, lbHeight + 1, groupWidth, 
                                 height - lbHeight - 1,
                                 groups
                                 )
      
      # group members listbox
      self.membersLbl = Label(self, groupWidth, lbHeight, "Members:")
      
      # prefill with members of the selected group
      if groups:
         members = groups[0].members
      else:
         members = []
      self.members = Listbox(self, groupWidth, lbHeight, width - groupWidth,
                             height - lbHeight - 1,
                             members
                             )

      # special stuff for key and menu processing
      self.__menu = \
         [
         (110, 'New address', self.newAddress),
         (101, 'Edit selection', self.editSelection),
         (100, 'Delete selection', self.deleteSelection),
         (103, 'new Group', self.newGroup),
         (109, 'add Member to group', self.addMemberToGroup),
         (113, 'Quit', self.quit),
         ]
      
      # add all keys in menu to local keytable
      self.keys = {}
      for key, label, func in self.__menu:
         self.keys[key] = func

      # add some special keys
      self.keys[10] = self.editSelection              # Enter key
      self.keys[self.KEY_DC] = self.deleteSelection   # delete key
      
      self.refresh()

   def _selectedGroupChanged(self, group):
      """
         Called by group list when the selected group changes: resets
         the group members list.
      """
      if group:
         self.members.setItems(group.members)
      else:
         self.members.setItems([])

   def __insertAddress(self, addr):
      """
         Inserts a new address object into its correct position in the list.
      """
      itemList = self.addresses.getItems()
      for cur, i in map(None, itemList, range(len(itemList))):
         if addr < cur:
            self.addresses.insert(i, addr)
            break
      else:
         self.addresses.append(addr)

   def __deleteAddress(self, addr):
      """
         Deletes an address from the list.
      """      
      # find its index and remove it
      i = self.addresses.getItems().index(addr)
      self.addresses.delete(i)

   def __insertGroup(self, group):
      """
         Inserts a new group into its correct position in the list.
      """
      itemList = self.groups.getItems()
      for item, i in map(None, itemList, range(len(itemList))):
         if item < group:
            self.groups.insert(i, group)
            break
      else:
         self.groups.append(group)

   def __deleteGroup(self, group):
      i = self.groups.getItems().index(group)
      self.groups.delete(i)

   def newAddress(self):
      # create a new address book entry
      addr = AddressEntry('', '', '', '')
      win = EditAddress(10, 10, addr)
      win.show(self)
      if win.result:
         main.addressBook[addr.nickname] = addr
         self.__insertAddress(addr)         

   def newGroup(self):
      """
         Create a new group entry.
      """
      group = GroupEntry('', '', [])
      win = EditGroup(10, 10, group)
      win.show(self)
      if win.result:
         main.addressBook[group.nickname] = group
         self.__insertGroup(group)

   def addMemberToGroup(self):
      """
         Adds the selected member to the selected group.
      """
      group = self.groups.selection()
      if not group:
         main.mainWin.showStatus('You must select a group')
         return
      
      member = self.addresses.selection()
      if not member:
         main.mainWin.showStatus('You must select an address')
         return
      
      if member.nickname in group.members:
         main.mainWin.showStatus('%s is already a member of %s' %
                                  (member.nickname, group.nickname)
                                 )
         return
      
      # store the member and write the group. Since the list is shared,
      # we only need to do this in the GUI.
      self.members.append(member.nickname)
      main.addressBook[group.nickname] = group      

   def editAddress(self):
      """
         Edits the currently selected item in the address list.
      """
      addr = self.addresses.selection()
      if not addr:
         main.mainWin.showStatus('You must select an address')
         return

      # preserve the nickname, which is the unique key
      key = addr.nickname

      win = EditAddress(10, 10, addr)
      win.show(self)
      if win.result:
         # If the nickname has not changed, we can just replace the existing 
         # item.  Otherwise, we have to delete the old and add the new.
         if addr.nickname == key:
            main.addressBook[key] = addr
         else:
            main.addressBook[addr.nickname] = addr
            del main.addressBook[key]

         # resort the item         
         self.__deleteAddress(addr)
         self.__insertAddress(addr)

   def editGroup(self):
      group = self.groups.selection()
      if not group:
         main.mainWin.showStatus('You must select a group')
         return
      
      # preserve the nickname
      nick = group.nickname
      
      win = EditGroup(10, 10, group)
      win.show(self)
      if win.result:
         # if the nickname changed, we need to remove the old entry
         if nick != group.nickname:
            del main.addressBook[nick]
         main.addressBook[group.nickname] = group

         # remove and reinsert the group into the list control
         self.__deleteGroup(group)
         self.__insertGroup(group)

   def editSelection(self):
      """
         Edits the selected item, regardless of whether the selection is
         in the addresses or groups list.
      """
      if self.getFocus() == self.addresses:
         self.editAddress()
      elif self.getFocus() == self.groups:
         self.editGroup()
      else:
         self.beep()
         main.mainWin.showStatus('You can not edit a group member')

   def deleteAddress(self):
      """
         Deletes the currently selected address.
      """
      addr = self.addresses.selection()
      if not addr:
         main.mainWin.showStatus('You must select an address')
         return
      
      if main.mainWin.queryYesNo('Delete address %s?' % addr.nickname):
         del main.addressBook[addr.nickname]
         self.__deleteAddress(addr)
         main.mainWin.showStatus('Address %s deleted' % addr.nickname)
   
   def deleteGroup(self):
      group = self.groups.selection()
      if not group:
         main.mainWin.showStatus('You must select a group')
         return
      
      if main.mainWin.queryYesNo('Delete group %s?' % group.fullName):
         del main.addressBook[group.nickname]
         i = self.groups.getItems().index(group)
         self.groups.delete(i)
         main.mainWin.showStatus('Group %s deleted' % group.fullName)
   
   def deleteMember(self):
      member = self.members.selection()
      if not member:
         main.mainWin.showStatus('You must select a group member')
         return
      
      # assuming that there _must_ be a group
      group = self.groups.selection()
      self.members.delete(group.members.index(member))
      
      # save the group
      main.addressBook[group.nickname] = group      

   def deleteSelection(self):
      if self.getFocus() == self.addresses:
         self.deleteAddress()
      elif self.getFocus() == self.groups:
         self.deleteGroup()
      else:
         self.deleteMember()

   def quit(self):
      self.terminate()

   def processEvent(self, evt):
      """
         Event handler for the window.
      """
      if isinstance(evt, KeyEvent):
         if evt.code == 24:                  # Ctrl-X
            main.menu(self.__menu, self)
            return 1
         elif self.keys.has_key(evt.code):
            self.keys[evt.code]()
            return 1
      
      return ModalWindow.processEvent(self, evt)
            
   
if __name__ == '__main__':
   from spug.curses.App import App
   class MyApp(App):      
      def processEvent(self, evt):
         if not App.processEvent(self, evt) and isinstance(evt, KeyEvent) and \
            evt.code == ord('q'):
            self.terminate()
   
   app = MyApp()
   EditAddress(10, 10).mainloop()

   